/*******************************************************************************
				Neoinput
				========

		Copyright (C) 2010 Paul Kocialkowski

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>

#include <linux/input.h>
#include <linux/uinput.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>

#include "server.h"
#include "input_read.h"
#include "input.h"

#include "input_preset.h"
#include "input_write.h"

int input=0;
char *input_device=NULL;
char *input_preset=NULL;

float speed_factor=1;

char *uinput_devices[3]={"/dev/input/uinput/", "/dev/uinput", "/dev/misc/uinput"};

void input_write_open(char *input_device)
{
	struct uinput_user_dev uinput_device;
	int i;

	/* Use uinput if nothing specified */
	if(input_device == NULL)
	{
		/* Try each uinput device */
		for(i=0 ; i < 3 ; i++)
		{
			input=open(uinput_devices[i], O_WRONLY | O_NONBLOCK);

			if(input < 0)
				continue;
			else
				break;
		}
	}
	else
		input=open(input_device, O_WRONLY);

	if(input < 0)
	{
		if(input_device == NULL)
			printf("Error while opening uinput.\n");
		else
			printf("Error while opening %s.\n", input_device);
		exit(1);
	}
	printf("input is %s\n", uinput_devices[i]);

	/*** BEGIN : Testing purpose only ***/
	/* Tu check les devices du preset pour voir quoi ioctl-er */

	ioctl(input, UI_SET_EVBIT, EV_KEY);
	ioctl(input, UI_SET_KEYBIT, BTN_LEFT);
	ioctl(input, UI_SET_KEYBIT, BTN_RIGHT);

	ioctl(input, UI_SET_EVBIT, EV_REL); 
	ioctl(input, UI_SET_RELBIT, REL_X);
	ioctl(input, UI_SET_RELBIT, REL_Y);

	ioctl(input, UI_SET_EVBIT, EV_SYN);

	//accel

	ioctl(input, UI_SET_KEYBIT, KEY_UP);
	ioctl(input, UI_SET_KEYBIT, KEY_DOWN);
	ioctl(input, UI_SET_KEYBIT, KEY_LEFT);
	ioctl(input, UI_SET_KEYBIT, KEY_RIGHT);
	ioctl(input, UI_SET_KEYBIT, KEY_RIGHTSHIFT);
	ioctl(input, UI_SET_KEYBIT, KEY_SPACE);

	if(input_device == NULL)
	{
		memset(&uinput_device, 0, sizeof(uinput_device));
		snprintf(uinput_device.name, UINPUT_MAX_NAME_SIZE, "uinput-sample");
		uinput_device.id.bustype = BUS_USB;
		uinput_device.id.vendor  = 0x1;
		uinput_device.id.product = 0x1;
		uinput_device.id.version = 1;

		write(input, &uinput_device, sizeof(uinput_device));

		ioctl(input, UI_DEV_CREATE);
	}

	/*** END : Testing purpose only ***/
}

void input_event_get(void)
{
	if(!sock_fd)
	{
		printf("Socket error.");
		exit(1);
	}

	struct input_event_light event_light;
	struct input_event event;

	__s32 event_light_value_old_0=0;
	__s32 event_light_value_old_1=0;

	int up_lock=0;
	int down_lock=0;
	int right_lock=0;
	int left_lock=0;
	int shift_lock=0;

	int pause_lock=1;

/*
	if(input_preset_value_bool("use_touch"))
	{
		if(strcmp(input_preset_value_get("touch_action"), "mouse") == 0)
			while(read(sock_fd, &event_light, sizeof(struct input_event_light)))
			{
				event.type=event_light.type;
				event.code=event_light.code;
				event.value=event_light.value;

				printf("--> %d %d %d\n", event_light.type, event_light.code, event_light.value);
				//input_event_write(event);
			}

	}
*/
	while(read(sock_fd, &event_light, sizeof(struct input_event_light)))
	{
		event.type=event_light.type;
		event.code=event_light.code;
		event.value=event_light.value;

		printf("--> %d %d %d\n", event_light.type, event_light.code, event_light.value);
		write(input, &event, sizeof(struct input_event));
	}
}

void input_write_close(void)
{
	if(input_device == NULL)
		ioctl(input, UI_DEV_DESTROY);
	close(input);
}
